package com.example.demo.service;

import com.example.demo.model.Todo;
import com.example.demo.repository.TodoRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class TodoService {

    @Autowired
    TodoRepository repository;

    public Flux<Todo> findAll() {
        return repository.findAll();
    }

    public Flux<Todo> findByCompletedFlag(boolean isCompleted) {
        return repository.findByCompleted(isCompleted);
    }

    /*
    This method is responsible for retrieving a paginated list of Todo objects from a data repository using Spring Data's reactive features. Here's a breakdown of the method:

    1) repository.findAllBy(pageable): This part of the code queries the data repository (repository) to retrieve a list of Todo objects based on the provided pageable parameter. The findAllBy method likely uses Spring Data MongoDB's built-in query capabilities to retrieve data in a pageable manner.

    2) .collectList(): This part of the code collects all the retrieved Todo objects into a Mono<List<Todo>>. The result is a Mono that emits a single list of Todo objects.

    3) .zipWith(repository.count()): In this step, the code combines the previously obtained Mono<List<Todo>> (from collectList) with another Mono<Long>, which represents the count of all Todo objects in the repository. The zipWith operator is used to combine the two Mono instances into a new Mono<Tuple2<List<Todo>, Long>>. This tuple contains both the list of Todo objects and the total count.

    4) .map(objects -> new PageImpl<>(objects.getT1(), pageable, objects.getT2())): Finally, the code maps the Tuple2 containing the list of Todo objects and the count into a Mono<Page<Todo>>. It constructs a PageImpl object, which is a Spring Data class used to represent paginated data. The PageImpl is initialized with the list of Todo objects (objects.getT1()), the provided pageable, and the total count (objects.getT2()). This resulting Mono emits a Page object representing the paginated results.
     */
    public Mono<Page<Todo>> findAllBy(Pageable pageable) {
        return repository.findAllBy(pageable)
                .collectList()
                .zipWith(repository.count())
                .map(objects -> new PageImpl<>(objects.getT1(), pageable, objects.getT2()));
    }
}
